@@ -155,9 +155,9 @@
     planet[1] = [planetList objectAtIndex:[planet2Popup indexOfSelectedItem]];
 
     if (!symbolFont)
-        symbolFont = [NSFont fontWithName:@"cenAstro" size:12.0];
+        symbolFont = [NSFont fontWithName:@"CenonAstro" size:12.0];
     if (!symbolFont)
-        symbolFont = [NSFont fontWithName:@"vhfAstro" size:12.0];
+        symbolFont = [NSFont fontWithName:@"vhfAstro"   size:12.0];
     if (!aspectList)
         aspectList = [NSMutableDictionary new];
 
