@@ -1,7 +1,7 @@
 /* EphemerisSearch.m
  * search Ephemeris data
  *
- * Copyright (C) 2004-2014 by Cenon GmbH
+ * Copyright (C) 2004-2015 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  2004-04-12
@@ -51,7 +51,7 @@
  * seconds/degree = (24*3600) / degPerDay
  */
 static unsigned secsPerDegree[O_ARRAYSIZE] =		// average seconds per degree of arc
- {    87660,     6552,   21120,   53929,    164885,	// sun - mars
+ {    87660,     6552,   21113,   53929,    164885,	// sun - mars
     1032628,  2582188, 7365729, 9185626, 217632242,	// jupiter - pluto
    16286522, 16286522,  775757,  775757,		// node, apogee
     4296370, 0, 0, 0, 0, 0				// chiron
@@ -77,6 +77,7 @@ static unsigned secsPerDegree[O_ARRAYSIZE] =		// average seconds per degree of a
 {
     progressTarget = target;
     progressAction = action;
+    progressFr = 0.0; progressTo = 1.0;
 }
 
 - (void)setTopocentricSearch:(BOOL)topoFlag lat:(float)lat lon:(float)lon elev:(int)elev
@@ -259,6 +260,14 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
     return value;
 }
 
+/* set range of progress indicator (default = full range = [0.0, 1.0])
+ */
+- (void)setProgressFr:(double)fr to:(double)to
+{
+    progressFr = fr;
+    progressTo = to;
+}
+
 /* binary search for a single aspect
  *
  * results =
@@ -276,7 +285,7 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
 {   NSMutableArray      *results = [NSMutableArray array];
     //int               years = [dateTo yearOfCommonEra] - [dateFr yearOfCommonEra] + 1;
     NSCalendarDate      *date;
-    int                 i, interval, minCycle, minRetro, loop = 0, loopCnt = 0;
+    int                 i, interval, minCycle, minRetro = 0, loop = 0, loopCnt = 0;
     int                 secCnt = 0;
     unsigned int        minDirect;
     EphObject           *objects;
@@ -326,7 +335,8 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
     /*    we use smallest section (cycle, retro) to split from start to end of range */
     minCycle = [self minutesCycleForAspect:obj1 :obj2 aspect:aspType];
     minCycle -= minCycle / 10;	// 10% tolerance
-    minRetro = [self minutesRetroForAspect:obj1 :obj2 aspect:aspType direct:&minDirect];
+    if ( !heliocentricSearch )
+        minRetro = [self minutesRetroForAspect:obj1 :obj2 aspect:aspType direct:&minDirect];
     interval = (minRetro > 0) ? Min(minCycle, minRetro-minRetro/5) : minCycle;
     sectionDates = [NSMutableArray array];
     for ( date = dateFr;					// interval steps through range
@@ -344,7 +354,7 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
                                longitude:topoLon latitude:topoLat elevation:topoElev topocentric:topocentricSearch
                             heliocentric:heliocentricSearch equatorial:equatorialSearch];
         /* add section date only, if retro changes or delta >= 360deg */
-        if ( !secCnt || min >= minCycle ||			// delta >= full cycle
+        if ( !secCnt || min >= minCycle-interval ||			// delta >= full cycle
              retro != ((objects[obj1].vLon < 0.0) ? YES : NO) )	// retro changes
         {
             [sectionDates addObject:date];
@@ -378,10 +388,11 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
         loop = 0; loopCnt = secCnt*20*2;	// 1st 50% of progress bar
         [turnDates addObject:dateFr];
         for (i=0; i<secCnt-1; i++)	// for each section
-        {   NSCalendarDate	*secDateFr, *secDateTo;
-            int			j;
-            AstroAspectValue	minV, v, maxV;
-            NSCalendarDate	*minDate, *maxDate, *date;
+        {   NSCalendarDate      *secDateFr, *secDateTo;
+            int                 j;
+            AstroAspectValue    minV, v, maxV;
+            NSCalendarDate      *minDate, *maxDate, *date;
+            double              val;
 
             secDateFr = [sectionDates objectAtIndex:i];		// start date of section
             secDateTo = [sectionDates objectAtIndex:i+1];	// end   date of section
@@ -397,8 +408,9 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
                 date = [minDate addTimeInterval:secs/2];
                 v = [self valueOfAspect:obj1 :obj2 aspect:aspType atDate:date];
 
-                [progressTarget performSelector:progressAction
-                                     withObject:[NSNumber numberWithFloat:(float)loop++/(float)loopCnt]];
+                val = progressFr + ((double)loop/(double)loopCnt) * (progressTo-progressFr);
+                [progressTarget performSelector:progressAction withObject:[NSNumber numberWithDouble:val]];
+                loop++;
 
                 if (secs < 30 || v.v == 0.0)	// ready !
                     break;
@@ -439,14 +451,15 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
     loop    = (retroAspect) ? secCnt*20 : 0;
     loopCnt = secCnt*20 * ((retroAspect) ? 2 : 1);	// 2nd 50% of progress bar
     for (i=0; i<secCnt-1; i++)	// for each section
-    {   NSCalendarDate	*secDateFr, *secDateTo;
-        int			j;
-        AstroAspectValue	minV, v, maxV;
-        NSCalendarDate		*minDate, *maxDate, *date;
-        BOOL			retro = NO;
+    {   NSCalendarDate      *secDateFr, *secDateTo;
+        int                 j;
+        AstroAspectValue    minV, v, maxV;
+        NSCalendarDate      *minDate, *maxDate, *date;
+        BOOL                retro = NO;
+        double              val;
 
-        secDateFr = [sectionDates objectAtIndex:i];	// start date of section
-        secDateTo = [sectionDates objectAtIndex:i+1];	// end   date of section
+        secDateFr = [sectionDates objectAtIndex:i];     // start date of section
+        secDateTo = [sectionDates objectAtIndex:i+1];   // end   date of section
 
         /* actual binary search inside section */
         minDate = secDateFr;
@@ -468,8 +481,9 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
                     /*skip = YES,*/ break;
             }
 
-            [progressTarget performSelector:progressAction
-                                 withObject:[NSNumber numberWithFloat:(float)loop++/(float)loopCnt]];
+            val = progressFr + ((double)loop/(double)loopCnt) * (progressTo-progressFr);
+            [progressTarget performSelector:progressAction withObject:[NSNumber numberWithDouble:val]];
+            loop++;
 
             if (secs < 30)	// ready !
                 break;
@@ -497,7 +511,7 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
         }
     }
     [progressTarget performSelector:progressAction
-                         withObject:[NSNumber numberWithFloat:1.0]];
+                         withObject:[NSNumber numberWithDouble:progressTo]];
 
     [progressTarget performSelector:@selector(setTitle:) withObject:nil];
 
@@ -538,10 +552,10 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
  */
 #define SIGNSIZE	15	// [degrees]
 - (NSDictionary*)searchWithFeatures:(NSDictionary*)features
-                           fromDate:(NSCalendarDate*)dateFrom toDate:(NSCalendarDate*)dateTo
+                           fromDate:(NSDate*)dateFrom toDate:(NSDate*)dateTo
                      searchInterval:(int)minutes
 {   NSMutableDictionary	*dict = [NSMutableDictionary dictionary];
-    NSCalendarDate	*date = dateFrom, *date0;
+    NSDate      *date = dateFrom, *date0;
     NSArray		*planets = [features objectForKey:@"planets"];
     int			planetCnt = [planets count];
     int			p, p1, planetIx[planetCnt], i;
@@ -556,7 +570,7 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
     NSArray		*dividers = [features objectForKey:@"dividers"];
     int			divCnt = [dividers count];
     double		target[planetCnt][planetCnt][divCnt],
-                        lastAngle [planetCnt][planetCnt][divCnt], closestAngle [planetCnt][planetCnt][divCnt];
+                lastAngle [planetCnt][planetCnt][divCnt], closestAngle [planetCnt][planetCnt][divCnt];
     double		lastMirror[planetCnt][planetCnt][divCnt], closestMirror[planetCnt][planetCnt][divCnt];
 
     if (!(featureEnterSigns || featureAspects || featureMirrors) || ![planets count])	// nothing to do
@@ -572,7 +586,7 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
     /* create dictionaries for planets */
     for (p=0; p<planetCnt; p++)
     {   NSMutableDictionary	*planetDict = [NSMutableDictionary dictionary];
-        NSString		*planet = [planets objectAtIndex:p];
+        NSString            *planet = [planets objectAtIndex:p];
 
         [dict setObject:planetDict forKey:planet];
         planetIx[p] = [ephemeris indexForObjectName:planet];
@@ -584,10 +598,10 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
         }
         if (featureAspects || featureMirrors)
         {
-            if (p < planetCnt-1)
+            if (p < planetCnt)
                 [planetDict setObject:[NSMutableDictionary dictionary] forKey:@"aspects"];
             dividers = [dividers sortedArrayUsingSelector:@selector(compare:)];
-            for (p1=0; p1<[planets count]; p1++)
+            for (p1=0; p1 < [planets count]; p1++)
                 for (i=0; i<divCnt; i++)
                 {
                     target[p][p1][i] = 360.0 / [[dividers objectAtIndex:i] floatValue];
@@ -599,7 +613,11 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
 
     /* search through range */
     while ([date compare:dateTo] != NSOrderedDescending)	// date <= dateTo
-    {   EphObject	*objects = [ephemeris objectsAtUTC:date];
+    {   //EphObject	*objects = [ephemeris objectsAtUTC:date];
+        EphObject	*objects = [ephemeris objectsAtUTC:date lat:topoLat lon:topoLon
+                                                elev:topoElev topocentric:topocentricSearch
+                                        heliocentric:heliocentricSearch
+                                          equatorial:equatorialSearch];
 
         /* entry of sign */
         if (featureEnterSigns)
@@ -624,7 +642,8 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
                     delta  = Diff(vPrev, v);				// delta interval
                     delta1 = Diff(vPrev, targetSign*signSize);		// delta target
                     dSec = minutes*60 * delta1 / delta  - (minutes*60.0);
-                    date0 = [date dateByAddingYears:0 months:0 days:0 hours:0 minutes:0 seconds:dSec];
+                    date0 = [date dateByAddingTimeInterval:dSec];
+                    //date0 = [date dateByAddingYears:0 months:0 days:0 hours:0 minutes:0 seconds:dSec];
 
                     //printf("sign changed %s: signNo[%d]=%d sign=%d", [[date0 description] cString], p, signNo[p], sign);
                     entryDict = [[dict objectForKey:[planets objectAtIndex:p]] objectForKey:@"entrySign"];
@@ -639,11 +658,11 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
         /* aspects */
         if (featureAspects || featureMirrors)
         {
-            for (p=0; p<planetCnt-1; p++)
+            for ( p=0; p < ((planetCnt > 1) ? planetCnt-1 : 1); p++ )           // obj1
             {   double  deg = objects[planetIx[p]].lon;
 
-                for (p1=p+1; p1<planetCnt; p1++)
-                {   double  deg1 = objects[planetIx[p1]].lon;
+                for (p1 = p + ((planetCnt>1) ? 1 : 0); p1 < planetCnt; p1++)    // obj2: TODO: add positions
+                {   double  deg1 = (planetCnt > 1) ? objects[planetIx[p1]].lon : 0.0;
 
                     if (featureAspects)
                     {   double  a = angle(deg, deg1);
@@ -656,23 +675,24 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
                             if ( is <= lastAngle[p][p1][i] )
                                 closestAngle[p][p1][i] = is;
                             else if (closestAngle[p][p1][i] < 3.0)
-                            {   double			v, delta, delta1;
-                                int			dSec;
-                                NSMutableDictionary	*entryDict;
-                                NSArray			*entry;
+                            {   double              v, delta, delta1;
+                                int                 dSec;
+                                NSMutableDictionary *entryDict;
+                                NSArray             *entry;
 
                                 /* get exact date: date += deltaTime * delta1 / delta */
                                 v = (is < closestAngle[p][p1][i]) ? is+target[p][p1][i] : is;
                                 delta  = Diff(closestAngle[p][p1][i], target[p][p1][i]) + Diff(v, target[p][p1][i]);
                                 delta1 = Diff(closestAngle[p][p1][i], target[p][p1][i]);
                                 dSec = minutes*60 * delta1 / delta  - (minutes*60.0);
-                                date0 = [date dateByAddingYears:0 months:0 days:0 hours:0 minutes:dSec/60 seconds:0];
+                                date0 = [date dateByAddingTimeInterval:dSec];
+                                //date0 = [date dateByAddingYears:0 months:0 days:0 hours:0 minutes:dSec/60 seconds:0];
                                 //printf("%s %s  v=%f delta=%f delta1 = %f\n",
                                 //[[date descriptionWithCalendarFormat:@"%Y%m%d %H:%M:%S"] cString],
                                 //[[date0 descriptionWithCalendarFormat:@"%Y%m%d %H:%M:%S"] cString],
                                 //v, delta, delta1);
 
-                                v = floor((a+target[p][p1][i]/2.0)/target[p][p1][i]) * target[p][p1][i];
+                                v = floor((a+target[p][p1][i]/2.0) / target[p][p1][i]) * target[p][p1][i];
                                 if (v >= 360.0) v = 0.0;
                                 //printf("a=%f v=%f target[p][p1][i] = %f\n", a, v, target[p][p1][i]);
                                 entry = [NSArray arrayWithObjects:[planets objectAtIndex:p1],
@@ -697,10 +717,10 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
                                 closestMirror[p][p1][i] = is;
                             else if (closestMirror[p][p1][i] < 360.0
                                      && Diff(closestMirror[p][p1][i], target[p][p1][i]) < 3.0)
-                            {   double			v, delta, delta1;
-                                int			dSec;
+                            {   double              v, delta, delta1;
+                                int                 dSec;
                                 NSMutableDictionary	*entryDict;
-                                NSArray			*entry;
+                                NSArray             *entry;
 
                                 /* get exact date: date += deltaTime * delta1 / delta */
                                 v = (is < closestMirror[p][p1][i]) ? is+target[p][p1][i] : is;
@@ -708,7 +728,8 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
                                          Diff(v, target[p][p1][i]);
                                 delta1 = Diff(closestMirror[p][p1][i], target[p][p1][i]);
                                 dSec = minutes*60 * delta1 / delta  - (minutes*60.0);
-                                date0 = [date dateByAddingYears:0 months:0 days:0 hours:0 minutes:dSec/60 seconds:0];
+                                date0 = [date dateByAddingTimeInterval:dSec];
+                                //date0 = [date dateByAddingYears:0 months:0 days:0 hours:0 minutes:dSec/60 seconds:0];
 
                                 v = floor((m+target[p][p1][i]/2.0)/target[p][p1][i])*target[p][p1][i];
                                 if (v >= 360.0) v = 0.0;
@@ -725,7 +746,8 @@ static unsigned hoursDirect[O_ARRAYSIZE] =	// intervals between retrograde in ho
             }
         } // end aspects
 
-        date = [date dateByAddingYears:0 months:0 days:0 hours:0 minutes:minutes seconds:0];
+        date = [date dateByAddingTimeInterval:minutes*60];
+        //date = [date dateByAddingYears:0 months:0 days:0 hours:0 minutes:minutes seconds:0];
     }
 
 NSLog(@"Search dict = %@", dict);
@@ -756,6 +778,7 @@ NSLog(@"Search dict = %@", dict);
     int                 i, divCnt = [dividers count], loop, loopCnt;
     double              target[divCnt], lastAngle[divCnt], closestAngle[divCnt];
     double              lastMirror[divCnt], closestMirror[divCnt];
+    //double            jd, jdTo, jdInterval = minutes / (24.0*60.0);
     NSArray             *entry;
     EphemerisFlags      ephFlags = EPHFLAG_LON; // what to calculate
     EphemerisObjects    ephObjIx[2];            // objects to calculate
@@ -777,11 +800,14 @@ NSLog(@"Search dict = %@", dict);
 
     loop = 0;
     loopCnt = ([dateTo timeIntervalSinceDate:date]/60.0) / minutes;
+    //jd   = [ephemeris julianDayForDate:date   bc:NO];
+    //jdTo = [ephemeris julianDayForDate:dateTo bc:NO];
+    //while ( jd <= jdTo )    // date <= dateTo
     while ([date compare:dateTo] != NSOrderedDescending)	// date <= dateTo
     {   NSAutoreleasePool   *pool = [NSAutoreleasePool new];
         EphObject           *objects;	// = [ephemeris objectsAtUTC:date];
 
-        objects = [ephemeris objectsAtJD:[ephemeris julianDayForDate:date bc:NO] flags:ephFlags
+        objects = [ephemeris objectsAtJD:/*jd*/[ephemeris julianDayForDate:date bc:NO] flags:ephFlags
                                  objects:ephObjIx cnt:ephObjCnt
                                longitude:topoLon latitude:topoLat elevation:topoElev topocentric:topocentricSearch
                             heliocentric:heliocentricSearch equatorial:equatorialSearch];
@@ -819,7 +845,8 @@ NSLog(@"Search dict = %@", dict);
                         {   NSLog(@"searchAspect, error in calculation: dSec = %d", dSec);
                             dSec = 0.0;
                         }
-                        date0 = [date dateByAddingYears:0 months:0 days:0 hours:0 minutes:dSec/60 seconds:0];
+                        //date0 = [date dateByAddingYears:0 months:0 days:0 hours:0 minutes:dSec/60 seconds:0];
+                        date0 = [date dateByAddingTimeInterval:dSec];
                     }
                     else	// FIXME: this should work with the signed angles as well
                     {
@@ -827,7 +854,8 @@ NSLog(@"Search dict = %@", dict);
                         delta  = Diff(closestAngle[i], target[i]) + Diff(v, target[i]);
                         delta1 = Diff(closestAngle[i], target[i]);
                         dSec = minutes*60 * delta1 / delta - (minutes*60.0);
-                        date0 = [date dateByAddingYears:0 months:0 days:0 hours:0 minutes:dSec/60 seconds:0];
+                        //date0 = [date dateByAddingYears:0 months:0 days:0 hours:0 minutes:dSec/60 seconds:0];
+                        date0 = [date dateByAddingTimeInterval:dSec];
                     }
 
                     /*printf("last=%f (%f) is=%f (%f) closest=%f t+is=%f t=%f\n", lastAngle[i], lsDelta, is, isDelta, closestAngle[i], is+target[i], target[i]);
@@ -871,7 +899,8 @@ NSLog(@"Search dict = %@", dict);
                         delta  = Diff(isSig, prSig);	// delta of time interval
                         delta1 = Abs(prSig);		// delta to target
                         dSec = - (minutes*60 * delta1 / delta) - (minutes*60);
-                        date0 = [date dateByAddingYears:0 months:0 days:0 hours:0 minutes:dSec/60 seconds:0];
+                        //date0 = [date dateByAddingYears:0 months:0 days:0 hours:0 minutes:dSec/60 seconds:0];
+                        date0 = [date dateByAddingTimeInterval:dSec];
                     }
                     else	// FIXME: this should work with the signed angles as well
                     {
@@ -879,7 +908,8 @@ NSLog(@"Search dict = %@", dict);
                         delta  = Diff(closestMirror[i], target[i]) + Diff(v, target[i]);
                         delta1 = Diff(closestMirror[i], target[i]);
                         dSec = minutes*60 * delta1 / delta  - (minutes*60.0);
-                        date0 = [date dateByAddingYears:0 months:0 days:0 hours:0 minutes:dSec/60 seconds:0];
+                        //date0 = [date dateByAddingYears:0 months:0 days:0 hours:0 minutes:dSec/60 seconds:0];
+                        date0 = [date dateByAddingTimeInterval:dSec];
                     }
                     v = floor((m+target[i]/2.0)/target[i])*target[i];	// target mirror angle
                     entry = [NSArray arrayWithObjects:planet1, planet2,
@@ -895,11 +925,15 @@ NSLog(@"Search dict = %@", dict);
             }
         }
 
-        [progressTarget performSelector:progressAction
-                             withObject:[NSNumber numberWithFloat:(float)loop++/(float)loopCnt]];
+        if ( loopCnt < 1000 || (loopCnt < 10000 && loop%10 == 0) || loop%100 == 0 )
+            [progressTarget performSelector:progressAction
+                                 withObject:[NSNumber numberWithFloat:(float)loop/(float)loopCnt]];
+        loop++;
 
         [date autorelease];
-        date = [[date dateByAddingYears:0 months:0 days:0 hours:0 minutes:minutes seconds:0] retain];
+        //date = [[date dateByAddingYears:0 months:0 days:0 hours:0 minutes:minutes seconds:0] retain];
+        date = [[date dateByAddingTimeInterval:minutes*60.0] retain];
+        //jd += jdInterval;
         [pool release];
     }
 
