@@ -199,7 +199,7 @@ NSPoint pointOnLineDistanceFromEnd(float dist, NSPoint p0, NSPoint p1)
     float           lineWidth = 1.0;
     BOOL            isACLine, wasACLine = NO;
     double          jd = vhfJulianDay(utc);	// julian day (reference -4712)
-    NSString        *astroFontName = (![(NSString*)Prefs_AstroFont length]) ? @"vhfAstro" : Prefs_AstroFont;
+    NSString        *astroFontName = (![(NSString*)Prefs_AstroFont length]) ? @"CenonAstro" : Prefs_AstroFont;
     NSFont          *symbolFont14 = [NSFont fontWithName:astroFontName size:14];
 
     [dateAxes release];
@@ -442,7 +442,7 @@ NSLog(@"1: lat=%f l1=%f -> p={%.2f %.2f}", lat, lon, p.x, p.y);
     int                 i, j, houseCnt = 12, n;
     float               lineWidth = 1.0;
     double              jd = vhfJulianDay(utc);	// julian day (reference -4712)
-    NSString            *astroFontName = (![(NSString*)Prefs_AstroFont length]) ? @"vhfAstro" : Prefs_AstroFont;
+    NSString            *astroFontName = (![(NSString*)Prefs_AstroFont length]) ? @"CenonAstro" : Prefs_AstroFont;
     NSFont              *symbolFont14 = [NSFont fontWithName:astroFontName size:14];
     NSFont              *systemFont14 = [NSFont systemFontOfSize:14];
     TopocentricCycle	*topo = [TopocentricCycle topocentricCycleAtDate:utc latitude:refLat longitude:refLon];
